==> pystan-2.5.0.2-cp34-cp34m-macosx_10_6_intel.macosx_10_9_intel.macosx_10_9_x86_64.macosx_10_10_intel.macosx_10_10_x86_64.whl: pystan/stan/src/test/unit/agrad/rev/internal/precomputed_gradients_test.cpp <==
#include <stan/agrad/rev/internal/precomputed_gradients.hpp>
#include <gtest/gtest.h>


// This is an example of how to use stan::agrad::precomputed_gradients()
TEST(StanAgradRevInternal, precomputed_gradients) {
  double value;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;
  stan::agrad::var x1(2), x2(3);
  stan::agrad::var y;
  
  value = 1;
  vars.resize(2);
  vars[0] = x1;
  vars[1] = x2;
  gradients.resize(2);
  gradients[0] = 4;
  gradients[1] = 5;

  EXPECT_NO_THROW(y = stan::agrad::precomputed_gradients(value, vars, gradients));
  EXPECT_FLOAT_EQ(value, y.val());

  std::vector<double> g;
  EXPECT_NO_THROW(y.grad(vars, g));
  ASSERT_EQ(2U, g.size());
  EXPECT_FLOAT_EQ(gradients[0], g[0]);
  EXPECT_FLOAT_EQ(gradients[1], g[1]);

  stan::agrad::recover_memory();
}


TEST(StanAgradRevInternal, precomputed_gradients_vari_no_independent_vars) {
  double value;
  std::vector<stan::agrad::vari *> varis;
  std::vector<double> gradients;
  
  value = 1;
  varis.resize(0);
  gradients.resize(0);
  EXPECT_NO_THROW(stan::agrad::precomputed_gradients_vari(value, varis, gradients));
  stan::agrad::precomputed_gradients_vari vari(value, varis, gradients);
  EXPECT_FLOAT_EQ(value, vari.val_);
  EXPECT_FLOAT_EQ(0, vari.adj_);
  EXPECT_NO_THROW(vari.chain());
}

TEST(StanAgradRevInternal, precomputed_gradients_vari_mismatched_sizes) {
  double value;
  std::vector<stan::agrad::vari *> varis;
  std::vector<double> gradients;

  value = 1;
  varis.resize(1);
  gradients.resize(2);
  EXPECT_THROW(stan::agrad::precomputed_gradients_vari(value, varis, gradients),
               std::invalid_argument);
}

TEST(StanAgradRevInternal, precomputed_gradients_vari) {
  double value;
  std::vector<stan::agrad::vari *> varis;
  std::vector<double> gradients;
  stan::agrad::vari x1(2), x2(3);
  
  value = 1;
  varis.resize(2);
  varis[0] = &x1;
  varis[1] = &x2;
  gradients.resize(2);
  gradients[0] = 4;
  gradients[1] = 5;
  EXPECT_NO_THROW(stan::agrad::precomputed_gradients_vari(value, varis, gradients));
  stan::agrad::precomputed_gradients_vari vari(value, varis, gradients);  
  EXPECT_FLOAT_EQ(value, vari.val_);
  EXPECT_FLOAT_EQ(0, vari.adj_);

  EXPECT_NO_THROW(vari.chain())
    << "running vari.chain() with no independent variables";
  EXPECT_FLOAT_EQ(value, vari.val_);
  EXPECT_FLOAT_EQ(0, vari.adj_);
  EXPECT_FLOAT_EQ(0, x1.adj_);
  EXPECT_FLOAT_EQ(0, x2.adj_);

  vari.init_dependent();
  EXPECT_NO_THROW(vari.chain())
    << "running vari.chain() with vari initialized as dependent variable";
  EXPECT_FLOAT_EQ(value, vari.val_);
  EXPECT_FLOAT_EQ(1, vari.adj_);
  EXPECT_FLOAT_EQ(gradients[0], x1.adj_);
  EXPECT_FLOAT_EQ(gradients[1], x2.adj_);
}

TEST(StanAgradRevInternal, precomputed_gradients_mismatched_sizes) {
  double value;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;
  
  value = 1;
  vars.resize(1);
  vars[0] = 0;
  gradients.resize(2);
  gradients[0] = 2;
  gradients[1] = 3;

  EXPECT_THROW(stan::agrad::precomputed_gradients(value, vars, gradients),
               std::invalid_argument);
  stan::agrad::recover_memory();
}

==> pystan-2.6.0.0-cp34-cp34m-macosx_10_6_intel.macosx_10_9_intel.macosx_10_9_x86_64.macosx_10_10_intel.macosx_10_10_x86_64.whl: pystan/stan/src/test/unit/agrad/rev/internal/precomputed_gradients_test.cpp <==
#include <stan/agrad/rev/internal/precomputed_gradients.hpp>
#include <gtest/gtest.h>


TEST(StanAgradRevInternal, precomputed_gradients) {
  double value;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;
  stan::agrad::var x1(2), x2(3);
  stan::agrad::var y;
  
  value = 1;
  vars.resize(2);
  vars[0] = x1;
  vars[1] = x2;
  gradients.resize(2);
  gradients[0] = 4;
  gradients[1] = 5;

  EXPECT_NO_THROW(y = stan::agrad::precomputed_gradients(value, vars, gradients));
  EXPECT_FLOAT_EQ(value, y.val());

  std::vector<double> g;
  EXPECT_NO_THROW(y.grad(vars, g));
  ASSERT_EQ(2U, g.size());
  EXPECT_FLOAT_EQ(gradients[0], g[0]);
  EXPECT_FLOAT_EQ(gradients[1], g[1]);

  stan::agrad::recover_memory();
}


TEST(StanAgradRevInternal, precomputed_gradients_vari_no_independent_vars) {
  double value = 1;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;

  stan::agrad::precomputed_gradients_vari vi(value, vars, gradients);
  EXPECT_FLOAT_EQ(value, vi.val_);
  EXPECT_FLOAT_EQ(0, vi.adj_);
  EXPECT_NO_THROW(vi.chain());
}

TEST(StanAgradRevInternal, precomputed_gradients_vari_mismatched_sizes) {
  double value;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;

  value = 1;
  vars.resize(1);
  gradients.resize(2);
  EXPECT_THROW(stan::agrad::precomputed_gradients_vari(value, vars, gradients),
               std::invalid_argument);
}

TEST(StanAgradRevInternal, precomputed_gradients_vari) {
  double value = 1;
  std::vector<stan::agrad::var> vars;
  stan::agrad::var x1(2), x2(3);
  vars.push_back(x1);
  vars.push_back(x2);

  std::vector<double> gradients;
  gradients.push_back(4);
  gradients.push_back(5);

  stan::agrad::precomputed_gradients_vari vi(value, vars, gradients);  
  EXPECT_FLOAT_EQ(value, vi.val_);
  EXPECT_FLOAT_EQ(0, vi.adj_);

  EXPECT_NO_THROW(vi.chain())
    << "running vi.chain() with no independent variables";
  EXPECT_FLOAT_EQ(value, vi.val_);
  EXPECT_FLOAT_EQ(0, vi.adj_);
  EXPECT_FLOAT_EQ(0, x1.vi_->adj_);
  EXPECT_FLOAT_EQ(0, x2.vi_->adj_);

  vi.init_dependent();
  EXPECT_NO_THROW(vi.chain())
    << "running vari.chain() with vari initialized as dependent variable";
  EXPECT_FLOAT_EQ(value, vi.val_);
  EXPECT_FLOAT_EQ(1, vi.adj_);
  EXPECT_FLOAT_EQ(gradients[0], x1.vi_->adj_);
  EXPECT_FLOAT_EQ(gradients[1], x2.vi_->adj_);
}

TEST(StanAgradRevInternal, precomputed_gradients_mismatched_sizes) {
  double value;
  std::vector<stan::agrad::var> vars;
  std::vector<double> gradients;
  
  value = 1;
  vars.resize(1);
  vars[0] = 0;
  gradients.resize(2);
  gradients[0] = 2;
  gradients[1] = 3;

  EXPECT_THROW(stan::agrad::precomputed_gradients(value, vars, gradients),
               std::invalid_argument);
  stan::agrad::recover_memory();
}
